/**
 *  zl 20220621 和地图相关的组件中 与map切换、获取、删除操作相关的部分都提取到这个对象
 *  map对象获取绑定的操作对象
 *  以混入的方式mixins提供给组件使用
 *  实现了map对象获取、加载监听、删除监听、销毁方法
 */

import mapEvent from "../_types/map-event";
import globalEvent from "@/common/_types/global-event";
import Mitt from "@/common/_utils/mitt";

export default {
  data() {
    return {
      map: null,
      webmap: null,
      mitt: new Mitt()
      // zl ？？？暂时没用
      // viewModel: null,
      // $t: null
    };
  },
  props: {
    mapTarget: String
  },
  // ？？？zl 需要后期测试
  // watch: {
  //   "props.mapTarget": {
  //     handler(newVal, oldVal) {
  //       if (newVal && oldVal && newVal !== oldVal) {
  //         // 多个map切换的时候，需要删除该组件与前一个map的图层绑定
  //         //callHook(this, "removed");
  //         if (mapEvent.$options.getMap(newVal)) {
  //           this.loadMap(newVal);
  //         }
  //       }
  //     },
  //     deep: true
  //   }
  // },
  mounted() {
    const targetName = this.getTargetName();

    if (mapEvent.getMap(targetName)) {
      this.loadMap(targetName);
    }
    // 添加map对象加载监听事件
    mapEvent.mitt.on("load-map", this.loadMapSucceed);
    // 添加map对象删除监听事件
    globalEvent.on("delete-map", this.deleteMapSucceed);
  },
  beforeDestroy() {
    mapEvent.mitt.off("load-map", this.loadMapSucceed);
    globalEvent.off("delete-map", this.deleteMapSucceed);
  },
  methods: {
    getTargetName() {
      /**
       * 便于区分存在多个map时，子组件对应的map的渲染；
       * map 和 webmap  的 props 属性是 target 其他组件都叫 mapTarget
       * 如果子组件包裹在 map 组件里面，若没有传 mapTarget, 则 targetName 直接取父元素的target 的值
       * 如果子组件和 map 同层级，且没有设置 mapTarget 时，则默认渲染到第一个 map 上
       */
      const selfParent = this.$parent;
      const parentTarget =
        selfParent &&
        selfParent.$options.name &&
        selfParent.$options.name.toLowerCase() === "kqwebmap" &&
        // @ts-ignore
        selfParent.target;
      // @ts-ignore
      return this.mapTarget || parentTarget || Object.keys(mapEvent.getAllMaps())[0];
    },
    loadMap(targetName) {
      this.map = mapEvent.getMap(targetName);
      // @ts-ignore
      this.webmap = mapEvent.getWebMap(targetName);
      //callHook(this, "loaded");
      // 控制与map组件同级的组件的显示加载
      this.$nextTick(() => {
        /**
         * @event loaded
         * @desc 组件加载渲染完成之后触发。
         */
        this.mitt.fire("loaded");
      });
    },
    loadMapSucceed(params) {
      const targetName = this.getTargetName();
      if (params.target === targetName) {
        this.loadMap(params.target);
      }
    },
    deleteMapSucceed(target) {
      const targetName = this.getTargetName();
      if (target === targetName) {
        //callHook(this, "removed");
        this.map = null;
        this.webmap = null;
        this.viewModel && (this.viewModel = null);
      }
    }
  }
};
// ？？？zl 需要后期测试
// function callHook(vm, hook) {
//   const { options } = vm.constructor;
//   options.mixins &&
//     options.mixins.forEach(mixin => {
//       mixin[hook] && mixin[hook].call(vm, vm.$options.name);
//     });
//   options[hook] && options[hook].call(vm, vm); // 调用子组件的生命周期
// }
