/**
 *  zl 20220817
 *  map对象获取绑定的操作对象
 */

import mapEvent from "../_types/map-event";
import { getCurrentInstance } from "vue";

export default {
  getTargetName(cb) {
    const { proxy } = getCurrentInstance();
    /**
     * 便于区分存在多个map时，子组件对应的map的渲染；
     * map 和 webmap  的 props 属性是 target 其他组件都叫 mapTarget
     * 如果子组件包裹在 map 组件里面，若没有传 mapTarget, 则 targetName 直接取父元素的target 的值
     * 如果子组件和 map 同层级，且没有设置 mapTarget 时，则默认渲染到第一个 map 上
     */
    const selfParent = proxy.$parent;
    const parentTarget =
      selfParent &&
      selfParent.$options.name &&
      selfParent.$options.name.toLowerCase() === "kqmapview" &&
      selfParent.$props.target;

    if (!parentTarget) {
      const _webmaps = Object.keys(mapEvent.getAllWebMap());
      if (_webmaps.length === 0) {
        mapEvent.mitt.on("load-map", e => {
          cb(e.eventTarget);
        });
      } else {
        cb(_webmaps[0]);
      }
    } else {
      cb(parentTarget);
    }
  },
  getWebMap(cb, targetName) {
    let webmap = mapEvent.getWebMap(targetName);
    if (webmap && webmap.map) {
      cb(webmap);
    } else {
      // 添加map对象加载监听事件
      // 清除事件的时间：在load-map执行后就立即清空了
      mapEvent.mitt.on("load-map", () => {
        webmap = mapEvent.getWebMap(targetName);
        if (webmap) {
          cb(webmap);
        }
      });
    }
  }
};
