import Mitt from "@/common/_utils/mitt";
import { mapViewUtils } from "@/common/_utils/gis-utils";
import { utils } from "@/common/_utils/gis-utils";
import { nextTick } from "vue";

export default class CompareMapViewModel extends Mitt {
  _props = null;
  webMap = null;
  constructor(mapTarget) {
    super();
    this._props = { mapTarget };
    // 事件绑定this
    this._loadedAsync = this._loadedAsync.bind(this);
    this._loadedSync = this._loadedSync.bind(this);    
    this.setLayerData = this.setLayerData.bind(this);
    this._registerEvents();
  }

  /**
   * 注册事件
   */
  _registerEvents() {  
    // mapview初始化结束监听 如果view已经存在了，mounted事件不会添加监听
    mapViewUtils.bindViewEvents("mounted", this._loadedAsync, this._props.mapTarget);  
    // 同步
    utils.getWebMap(this._props.mapTarget, this._loadedSync);
  }

  _loadedAsync(e) {
    // 如果图层变化事件是自己触发的 则不继续执行 避免死循环
    if (e.eventTarget && e.eventTarget === this) {
      return;
    }
    this.webMap = e.target;
    this.setLayerData();
    this.getWebMap();
  }

  _loadedSync(e) {
    this.webMap = e;
    this.setLayerData();
  }  

  // 设置图层树数据
  setLayerData() {    
    let layerCheckedKeys = this.webMap._layerCheckedKeys;
    let layerCheckedList = this.webMap._layerCheckedList;
    nextTick(() => {
      this.fire("setLayerData", { layerCheckedKeys, layerCheckedList });
    });
  }

  // 二维勾选图层逻辑
  setLayerVisible(value, visible) {
    this.webMap.setLayerOpacityStateById(value, visible ? 1 : 0);    
  }

  /**
   * 获取webmap
   */
  getWebMap() {
    this.fire("getWebMap", { webMap: this.webMap });
  }
}
