export default class MapCompare {
  constructor(currentMapId, mainMapView) {
    // 对比地图
    this.mapCompareConfig = {};
    // 主要的地图
    this.mainMapView = mainMapView;
    // 当前操作图层ID
    this.currentMapId = currentMapId;
    // 当前地图已关闭二、四窗口
    this.isClose = false;
    // 当前地图的右键菜单
    this.checkedMenu = null;
    // 鼠标移动多少会触发一次同步, 值越小越灵敏，请求服务越多
    this.moveDistance = 30;
    // 是否在移动
    this.isMove = false;
    // 转屏幕坐标
    this.project = this.mainMapView.map.latLngToLayerPoint.bind(this.mainMapView.map);
    // 初始增加主地图
    this.mapCompareConfig[mainMapView.target] = {
      mapView: mainMapView,
      isSynchro: true, // 是否同步
      marker: null, // 鼠标移动的圆标示
      startPoint: null // 地图开始移动的位置
    };
  }
}

// 增加主地图事件
MapCompare.prototype.addMainMapEvent = function () {
  this.mainMapView.map.on("mouseover", this.onMousedown, this);
  // 鼠标移动事件
  this.mainMapView.map.on("mousemove", this.onMousemove, this);
  // 鼠标移出事件
  this.mainMapView.map.on("mouseout", this.onMouseout, this);
  // 鼠标按下事件
  this.mainMapView.map.on("mousedown", this.onMousedown, this);
  // 地图移动事件
  this.mainMapView.map.on("move", this.onMove, this);
  // 地图移动开始事件
  this.mainMapView.map.on("movestart", this.onMovestart, this);
  // 地图移动结束事件
  this.mainMapView.map.on("moveend", this.onMoveend, this);
  // 鼠标缩放事件
  this.mainMapView.map.on("zoomend", this.onZoomend, this);
};

// 移出主地图事件
MapCompare.prototype.removeMainMapEvent = function () {
  this.mainMapView.map.off("mouseover", this.onMousedown, this);
  // 鼠标移动事件
  this.mainMapView.map.off("mousemove", this.onMousemove, this);
  // 鼠标移出事件
  this.mainMapView.map.off("mouseout", this.onMouseout, this);
  // 鼠标按下事件
  this.mainMapView.map.off("mousedown", this.onMousedown, this);
  // 地图移动事件
  this.mainMapView.map.off("move", this.onMove, this);
  // 地图移动开始事件
  this.mainMapView.map.off("movestart", this.onMovestart, this);
  // 地图移动结束事件
  this.mainMapView.map.off("moveend", this.onMoveend, this);
  // 鼠标缩放事件
  this.mainMapView.map.off("zoomend", this.onZoomend, this);
};

MapCompare.prototype.addMap = function (compMapView) {
  compMapView.map.on("mouseover", this.onMousedown, this);
  // 鼠标移动事件
  compMapView.map.on("mousemove", this.onMousemove, this);
  // 鼠标移出事件
  compMapView.map.on("mouseout", this.onMouseout, this);
  // 鼠标按下事件
  compMapView.map.on("mousedown", this.onMousedown, this);
  // 地图移动事件
  compMapView.map.on("move", this.onMove, this);
  // 地图移动开始事件
  compMapView.map.on("movestart", this.onMovestart, this);
  // 地图移动结束事件
  compMapView.map.on("moveend", this.onMoveend, this);
  // 鼠标缩放事件
  compMapView.map.on("zoomend", this.onZoomend, this);

  this.mapCompareConfig[compMapView.target] = {
    mapView: compMapView,
    isSynchro: true,
    marker: null,
    startPoint: null
  };
  compMapView.map.setView(this.mainMapView.map.getCenter(), this.mainMapView.map.getZoom());
};

// 鼠标按下事件
MapCompare.prototype.onMousedown = function (e) {
  let mapId = e.target._container.id;
  if (this.currentMapId !== mapId) {
    this.currentMapId = mapId;
  }
};

// 地图移动事件
MapCompare.prototype.onMove = function (e) {
  let mapId = e.target._container.id;
  if (this.currentMapId === mapId) {
    this.isMove = false;
  }
  if (this.mapCompareConfig[mapId].isSynchro) {
    let movePoint = this.project(e.target.getCenter());
    if (this._isTrueDistanceMove(this.mapCompareConfig[mapId].startPoint, movePoint)) {
      this.mapCompareConfig[mapId].startPoint = movePoint;
      this.mapMove(e, mapId);
    }
  }
};
// 地图移动开始事件
MapCompare.prototype.onMovestart = function (e) {
  let mapId = e.target._container.id;
  if (this.currentMapId === mapId) {
    this.isMove = true;
  }
  this.mapCompareConfig[mapId].startPoint = this.project(e.target.getCenter());
};
// 地图移动结束事件
MapCompare.prototype.onMoveend = function (e) {
  let mapId = e.target._container.id;
  if (this.currentMapId === mapId) {
    this.isMove = false;
  }
  if (this.mapCompareConfig[mapId].isSynchro) {
    this.mapMove(e, mapId);
  }
};
// 鼠标缩放事件
MapCompare.prototype.onZoomend = function (e) {
  let mapId = e.target._container.id;
  if (this.currentMapId === mapId) {
    this.isMove = false;
  }
  if (this.mapCompareConfig[mapId].isSynchro) {
    this.mapMove(e, mapId);
  }
};
//鼠标移动事件
MapCompare.prototype.onMousemove = function (e) {
  let lat = e.latlng.lat.toFixed(4);
  let lng = e.latlng.lng.toFixed(4);
  let mapId = this.currentMapId;
  // 给不同地图添加marker
  if (!this.isClose) {
    if (mapId === this.mainMapView.target) {
      this.moveMarker(lat, lng, "map2");
      this.moveMarker(lat, lng, "map3");
      this.moveMarker(lat, lng, "map4");
    } else if (mapId === "map2") {
      this.moveMarker(lat, lng, this.mainMapView.target);
      this.moveMarker(lat, lng, "map3");
      this.moveMarker(lat, lng, "map4");
    } else if (mapId === "map3") {
      this.moveMarker(lat, lng, this.mainMapView.target);
      this.moveMarker(lat, lng, "map2");
      this.moveMarker(lat, lng, "map4");
    } else if (mapId === "map4") {
      this.moveMarker(lat, lng, this.mainMapView.target);
      this.moveMarker(lat, lng, "map2");
      this.moveMarker(lat, lng, "map3");
    }
  }
};
// 鼠标移出事件
MapCompare.prototype.onMouseout = function (e) {
  for (let mapId in this.mapCompareConfig) {
    if (this.mapCompareConfig[mapId]["marker"] !== null) {
      this.mapCompareConfig[mapId]["marker"].remove();
      this.mapCompareConfig[mapId]["marker"] = null;
    }
  }
};

// 地图移动事件
MapCompare.prototype.mapMove = function (e, mapId) {  
  if (this.currentMapId === mapId) {
    let map = e.target;
    for (let name in this.mapCompareConfig) {
      if (name !== mapId && this.mapCompareConfig[name].isSynchro) {
        this.mapCompareConfig[name].mapView.map.setView(map.getCenter(), map.getZoom());
      }
    }
  }
};

// 创建鼠标移动的圆标示
MapCompare.prototype.moveMarker = function (lat, lng, mapName) {
  for (let name in this.mapCompareConfig) {
    if (name === mapName) {
      if (this.mapCompareConfig[mapName].marker === null) {
        this.mapCompareConfig[mapName].marker = L.circleMarker([lat, lng], {
          stroke: true,
          color: "#aaaaaa",
          weight: 1,
          opacity: 1,
          fillColor: "#0087FF",
          fillOpacity: 0.6,
          radius: 15
        });
        this.mapCompareConfig[mapName].marker.addTo(this.mapCompareConfig[mapName].mapView.map);
      }
      this.mapCompareConfig[mapName].marker.setLatLng([Number(lat), Number(lng)]);
    }
  }
};

/**
 * 判断是否大于或等于有效触发同步移动的距离
 * @param {*} startPoint
 * @param {*} movePoint
 * @returns
 */
MapCompare.prototype._isTrueDistanceMove = function (startPoint, movePoint) {
  if (this.moveDistance === 0) {
    return true;
  }
  if (startPoint === null) {
    return false;
  }
  let distance = Math.sqrt(Math.pow(startPoint.x - movePoint.x, 2) + Math.pow(startPoint.y - movePoint.y, 2));
  if (distance >= this.moveDistance) {
    return true;
  }
  return false;
};

// 关闭对比销毁数据
MapCompare.prototype.destroy = function () {
  // 销毁地图对象
  for (let target in this.mapCompareConfig) {
    if (target !== this.mainMapView.target) {
      delete this.mapCompareConfig[target];
    }
  }
  // 当前操作图层ID
  this.currentMapId = this.mainMapView.target;
  this.isMove = false;
  // 移出主地图事件
  this.removeMainMapEvent();
};
