/**
 * 202207 zl
 * view支持的场景
 * 1.支持空地图，引用场景是显示前端实时导入数据。参数serviceUrl都为空
 * 2.支持传入单个kq服务，地图信息和图层从服务中动态获取。必须参数serverUrl和serviceId
 * 3.多个kq服务，地图信息和图层从第一个服务中动态获取。？？？暂不考虑
 * 4.支持portal创建的地图定制应用
 * 5.支持kqgis server的聚合服务
 */

import L from "leaflet";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet/leaflet.css";
import "@/../public/libs/kqwebclient/leaflet/kqwebclient-leaflet-es6.min";
import WebMapService from "@/common/_utils/webmapservice";
import { formatKQMapInfo } from "@/common/_utils/util";
import MapDraw from "@/leaflet/mapview/MapDraw";
import MapControl from "@/leaflet/mapview/MapControl";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/Leaflet.Editable/Leaflet.Editable";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet.pm/leaflet.pm.min";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet.pm/leaflet.pm.css";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet-measure-path-master/leaflet-measure-path.css";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet-measure-path-master/leaflet-measure-path";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/Leaflet.MagnifyingGlass/leaflet.magnifyingglass.css";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/Leaflet.MagnifyingGlass/leaflet.magnifyingglass";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/History";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/Leaflet.GraphicScale/Leaflet.GraphicScale.min.js";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/Leaflet.GraphicScale/Leaflet.GraphicScale.min.css";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet.minimap/Control.MiniMap.min.css";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet.minimap/Control.MiniMap.min";
import "@/../public/libs/kqwebclient/leaflet/3rd-libs/leaflet-polygon.fillPattern/leaflet-polygon.fillPattern.js";
import { KqMessage } from "@/common";
import globalEvent from "@/common/_types/global-event";

export default class MapViewViewModel extends L.Evented {
  map = null; // 地图对象
  mapDraw = null; // 绘图、测量操作类
  mapControl = null; // 卷帘、滤镜图层类
  target = ""; // 地图对象初始化的div的id
  serviceId = ""; // 地图服务名
  serviceUrl = ""; //kq地图服务地址
  accessToken = "";
  ////////////////???zl 合并
  webMapInfo = null; // 地图服务信息
  mapOptions = {}; //地图初始化外部传参 优先级高于从服务获取的信息
  // /////////////////

  _props = null;
  _taskID = null; // 唯一值

  _defaultShowLayerNames = []; // 默认勾选的图层
  _isShowAllLayers = true; // 默认勾选参数defaultShowLayerNames为空时 以参数isShowAllLayers为准

  _mapLayerListData = []; // 图层列表数据
  _mapLayerTreeData = []; // 图层树数据

  _zIndex = 200; // 图层的索引
  _layerCheckedList = []; // 选中的图层列表
  _layerCheckedKeys = []; // 选中的图层id列表

  _frontLayer = null; // 置顶的图层
  _frontLayerOriginalIndex; // 置顶图层原始的zIndex

  _layers = new Map(); // 图层对象集合

  _mapBaseLayerData = new Map(); // 切换底图图层数据

  _mapOnLineLayerData = new Map(); //在线底图
  _mapOnLineLayerGroup = null;

  constructor(serviceId, props) {
    super();
    this.serviceId = serviceId;
    this.mapOptions = props.mapOptions || {};
    this.target = props.target || "map";
    this.serviceUrl = props.serviceUrl;
    this.accessToken = props.token;

    this._props = props;
    this._defaultShowLayerNames = props.defaultShowLayerNames ? props.defaultShowLayerNames : [];

    this._isShowAllLayers = typeof props.isShowAllLayers !== "undefined" ? props.isShowAllLayers : true;

    this.webMapInfo = props.webMapInfo;

    this._initWebMap();
  }
  // 初始化地图
  _initWebMap() {
    this.clearWebMap();

    this._taskID = new Date();

    if (!this.webMapInfo) {
      if (this.serviceUrl) {
        this._getMapInfo();
      } else {
        this._after();
      }
    } else {
      this._after();
    }
  }
  // 获取服务信息
  _getMapInfo() {
    if (!(this.serviceId && this.serviceId !== "")) {
      throw "传入参数serviceId为空，也没有传入webMapInfo信息！";
    }

    let webMapService = new WebMapService(this.serviceId, this._props);

    this.serviceUrl = this.serviceUrl && webMapService.handleServerUrl(this.serviceUrl);
    webMapService
      .getMapInfo()
      .then(
        mapInfo => {
          if (mapInfo.resultcode === "error") {
            console.error(mapInfo);
            KqMessage({ message: mapInfo.message, grouping: true, type: "error", duration: 6000 });
            return;
          }
          // 存储地图的名称以及描述等信息，返回给用户
          this.webMapInfo = formatKQMapInfo(
            mapInfo,
            this.serviceUrl,
            this.serviceId,
            this.accessToken,
            this._defaultShowLayerNames,
            this._isShowAllLayers
          );

          this._after(mapInfo);
        },
        error => {
          throw error;
        }
      )
      .catch(error => {
        console.error(error);
        KqMessage({ message: error, grouping: true, type: "error", duration: 6000 });
      });
  }
  // 执行获取服务信息后的操作
  _after() {
    if (this.webMapInfo) {
      this._mapLayerListData = this.webMapInfo.layers_show;
      this._mapLayerTreeData = this.webMapInfo.layerTreeData;

      if (this.mapOptions) {
        this.webMapInfo = { ...this.webMapInfo, ...this.mapOptions };
      }
    }

    this._createMap();

    this._addDefaultLayer();

    this._showDefaultBaseLayer();

    this.mapDraw = new MapDraw(this);
    this.mapControl = new MapControl(this);
    // 上一视图下一视图使用
    this.history = new L.HistoryControl({ map: this.map });
    this.magnifyingGlass = L.magnifyingGlass;
    // 执行view初始化成功事件
    this._mapLoated();
    // 触发mapview初始化完成事件
    this.fire("mounted");
  }
  // 创建map对象
  _createMap() {
    if (this.webMapInfo) {
      let crs = this._handleMapCrs();
      // 初始化 map
      this.map = L.map(this.target, {
        center: this.webMapInfo.center,
        zoom: this.webMapInfo.zoom,
        crs: crs,
        maxZoom: this.webMapInfo.maxzoom,
        minZoom: this.webMapInfo.minzoom,
        zoomControl: typeof this.mapOptions.zoomControl !== "undefined" ? this.mapOptions.zoomControl : true,
        editable: true // 测量编辑功能中需要开启这个属性
      });

      if (this.webMapInfo.minzoom === 0 && !this.mapOptions.zoom) {
        this.map.fitBounds(this.webMapInfo.bounds84);
      }
    } else {
      // 不传map信息(webMapInfo) 也不传服务url(serviceUrl)时，初始化默认地图
      this.map = L.map(this.target, {
        center: this.mapOptions.center || [34, 120],
        zoom: this.mapOptions.zoom || 6,
        crs: this.mapOptions.crs || L.CRS.EPSG3857,
        maxZoom: this.mapOptions.maxZoom || 22,
        minZoom: this.mapOptions.minZoom || 0,
        zoomControl: typeof this.mapOptions.zoomControl !== "undefined" ? this.mapOptions.zoomControl : true,
        editable: true // 测量编辑功能中需要开启这个属性
      });
    }
  }
  // 执行view初始化成功事件
  _mapLoated() {
    // 地图初始化成功后执行 mapview存储
    globalEvent.setWebMap(this.target, "leaflet", this);
    // 地图初始化成功后执行 地图加载完成事件
    globalEvent.fire("load-map", { eventTarget: this.target });
    // 触发结束后清空事件
    globalEvent.off("load-map");
  }

  /**
   * 设置默认显示的图层
   * @return 是否做默认图层显示、隐藏变化操作
   */
  _addDefaultLayer() {
    if (this.webMapInfo && this.webMapInfo.layers_show && this.webMapInfo.layers_show.length > 0) {
      let ids = [];
      this.webMapInfo.layers_show.forEach(item => {
        if (item.visible) {
          ids.push(item.guid);
        }
      });

      this.setLayerVisibleStateByIds(ids, true);
      return true;
    } else {
      return false;
    }
  }

  /**
   * 处理地图的空间参考
   */
  _handleMapCrs() {
    let { spatialReference, rect } = this.webMapInfo;
    // leaflet除了自带的几个空间参考 其他都得使用proj4声明
    if (!(spatialReference.epsg === "4326" || spatialReference.epsg === "3857")) {
      proj4.defs("EPSG:" + spatialReference.epsg, spatialReference.proj4);
    }
    let mapBounds = L.bounds([rect.minx, rect.miny], [rect.maxx, rect.maxy]);
    return new L.Proj.CRS("EPSG:" + spatialReference.epsg, {
      origin: this.webMapInfo.origin, // 中心原点
      resolutions: this.webMapInfo.resolutions, // 分辨率
      bounds: mapBounds
    });
  }
  _changeLayer(layerData, id, visible) {
    let l = this._getLayer(layerData);
    if (!l) return;

    if (visible) {
      if (!this.map.hasLayer(l)) {
        this.map.addLayer(l);
        this._layerCheckedList.push(layerData);
        this._layerCheckedKeys.push(layerData.guid);
      }
    } else {
      this.map.removeLayer(l);
      for (let n = 0; n < this._layerCheckedList.length; n++) {
        if (this._layerCheckedList[n].guid === id) {
          this._layerCheckedList.splice(n, 1);
          this._layerCheckedKeys.splice(n, 1);
          break;
        }
      }
    }
    layerData.visible = visible;
  }
  _getLayer(layerConfig) {
    if (!this._layers.has(layerConfig.guid)) {
      this._layers.set(layerConfig.guid, this._createLayer(layerConfig));
    }
    return this._layers.get(layerConfig.guid);
  }
  _createLayer(layerConfig) {
    let layer = null;
    switch (layerConfig["serverType"].toLowerCase()) {
      case "kqserver":
        layer = L.KqGIS.tileMapLayer(layerConfig.url, {
          layerIds: layerConfig.id,
          ua_token: layerConfig.accessToken,
          zIndex: layerConfig.zIndex
        });
        break;
      case "geojson":
        layer = L.geoJSON(layerConfig.data, {
          style: function (feature) {
            return { color: feature.properties.color };
          }
        });
        break;
      case "wmts": {
        let params = {
          layer: layerConfig["layer"],
          tilematrixSet: layerConfig["tilematrixSet"] === undefined ? "default028mm" : layerConfig["tilematrixSet"],
          style: layerConfig["style"] === undefined ? "default" : layerConfig["style"],
          format: layerConfig["format"] === undefined ? "image/jpgpng" : layerConfig["format"],
          transparent: layerConfig["transparent"] === undefined ? true : layerConfig["transparent"],
          zoomOffset:
            layerConfig["zoomOffset"] === undefined ? this.mapViewParams.zoomOffset : Number(parames["zoomOffset"])
        };
        if (layerConfig["subdomains"] !== undefined) {
          params["subdomains"] = layerConfig["subdomains"];
        }
        let url = layerConfig["url"];
        if (layerConfig["token"] !== undefined) {
          url += "?" + layerConfig["token"];
        }
        layer = window.L.KqGIS.wmtsLayer(url, params);
        break;
      }
      case "wms":
        break;
      case "tianditu":
        layer = window.L.KqGIS.tiandituTileLayer({ layerType: layerConfig.layerType, isDegree: layerConfig.isDegree });
        break;
      case "baidu":
        layer = window.L.KqGIS.baiduTileLayer({ layerType: layerConfig.layerType });
        break;
      case "gaode":
        layer = window.L.KqGIS.gaodeTileLayer({ layerType: layerConfig.layerType });
        break;
      case "tencent":
        layer = window.L.KqGIS.tencentTileLayer({ layerType: layerConfig.layerType });
        break;
      case "bing":
        layer = window.L.KqGIS.bingTileLayer({ imagerySet: layerConfig.layerType });
        break;
      case "geoq":
        layer = window.L.KqGIS.geoqTileLayer({ layerType: layerConfig.layerType });
        break;
      case "arcgisonline":
        layer = window.L.KqGIS.arcgisTileLayer({ layerType: layerConfig.layerType });
        break;
      case "mapbox":
        layer = window.L.KqGIS.mapboxTileLayer({ layerType: layerConfig.layerType });
        break;
      case "osm":
        layer = window.L.KqGIS.osmTileLayer({ layerType: layerConfig.layerType });
        break;
    }
    return layer;
  }
  _showDefaultBaseLayer() {
    if (!this._props.baseLayerInfo) return;
    for (let item in this._props.baseLayerInfo) {
      if (this._props.baseLayerInfo[item].visible === true) {
        this.showBaseLayer(item, true);
      }
    }
  }
}
MapViewViewModel.prototype.destroy = function () {
  // 销毁mapevent中存的对象
  globalEvent.removeWebMap(this.target);
  // 删除所有事件监听
  this.off();
  this.clearWebMap();
};
/**
 * 清除地图对象
 */
MapViewViewModel.prototype.clearWebMap = function () {
  if (this.map) {
    this.map.remove();
    this.map = null;
    this.center = null;
    this.zoom = null;
  }
};
// 获取所有图层原数据（包括adddata组件的临时图层数据，不包括底图baselayer）
MapViewViewModel.prototype.getLayerData = function () {
  return JSON.parse(JSON.stringify(this.webMapInfo.layers_show));
};
// 获取所有图层的key（包括adddata组件的临时图层数据，不包括底图baselayer）
MapViewViewModel.prototype.getLayerDataKey = function () {
  let keys = [];
  this.webMapInfo.layers_show.forEach(item => {
    keys.push(item.guid);
  });
  return keys;
};
// 获取所有图层的树形结构数据
MapViewViewModel.prototype.getLayerTreeData = function () {
  return JSON.parse(JSON.stringify(this._mapLayerTreeData));
};
// 获取所有显示状态图层的数据 ???是否排除临时图层
MapViewViewModel.prototype.getLayerCheckedList = function () {
  return JSON.parse(JSON.stringify(this._layerCheckedList));
};
// 获取所有显示状态图层的数据的key
MapViewViewModel.prototype.getLayerCheckedListKey = function () {
  return JSON.parse(JSON.stringify(this._layerCheckedKeys));
};
// 获取渲染地图的比例尺
MapViewViewModel.prototype.getScale = function () {
  return JSON.parse(JSON.stringify(this.webMapInfo.scale));
};
// 获取mapview渲染的数据
MapViewViewModel.prototype.getWebMapInfo = function () {
  return JSON.parse(JSON.stringify(this.webMapInfo));
};
// 获取mapview渲染的数据
MapViewViewModel.prototype.getBaseLayerInfo = function () {
  return JSON.parse(JSON.stringify(this._props.baseLayerInfo));
};

/**
 * 返回底图图层数据
 * @returns
 */
MapViewViewModel.prototype.getMapBaseLayerData = function () {
  return this._mapBaseLayerData;
};

/**
 * 设置多个图层的显示与隐藏 且控制树的勾选
 * @param ids 图层的GUID数组
 * @param visible true, false
 * @param eventTarget 触发方法的对象。该方法有执行其他监听事件，为防止方法死循环，该参数会传给监听事件
 */
MapViewViewModel.prototype.setLayerVisibleStateByIds = function (ids, visible, eventTarget) {
  if (!(ids && ids.length > 0)) return;

  // 是否有变化
  let isReset = false;

  for (let z = 0; z < ids.length; z++) {
    for (let i = 0; i < this._mapLayerListData.length; i++) {
      if (this._mapLayerListData[i].guid === ids[z]) {
        this._changeLayer(this._mapLayerListData[i], ids[z], visible);
        isReset = true;
      }
    }
  }
  // 触发layer变化事件
  if (isReset) this.fire("resetLayerDatas", { eventTarget });
};

/**
 * 设置当前鼠标指针
 * @param name 路径或者类型名称
 */
MapViewViewModel.prototype.setMapCursor = function (name) {
  document.getElementById(this.target).style.cursor = name;
};

/**
 * 设置透明度
 * @param guid 当前图层的guid
 * @param opacity 0-1之间的数
 */
MapViewViewModel.prototype.setLayerOpacityStateById = function (guid, opacity) {
  for (let i = 0; i < this._mapLayerListData.length; i++) {
    if (this._mapLayerListData[i].guid === guid) {
      this._getLayer(this._mapLayerListData[i]).setOpacity(opacity);
      this._mapLayerListData[i].opacity = opacity;
      break;
    }
  }
};

/**
 * 图层置顶
 * @param guid 图层的guid
 * @param isFront 置顶与还原（true, false）
 */
MapViewViewModel.prototype.setLayerBringToFrontById = function (guid, isFront) {
  if (!isFront) {
    // 还原置顶
    if (this._frontLayer !== null && this._frontLayer.visible && this._frontLayer.isFront) {
      this._frontLayer.isFront = false;
      this._frontLayer.zIndex = this._frontLayerOriginalIndex;
      let l_layer = this._getLayer(this._frontLayer);
      l_layer.setZIndex(this._frontLayer.zIndex);
      this._frontLayer = null;
    }
  } else {
    // 置顶
    for (let i = 0; i < this._mapLayerListData.length; i++) {
      if (this._mapLayerListData[i].guid === guid && this._mapLayerListData[i].visible) {
        // 如果已经有置顶图层 先把已经是置顶的图层还原 再继续操作
        if (this._frontLayer !== null && this._frontLayer.isFront) {
          this._frontLayer.isFront = false;
          this._frontLayer.zIndex = this._frontLayerOriginalIndex;
          let l_layer = this._getLayer(this._frontLayer);
          l_layer.setZIndex(this._frontLayer.zIndex);
        }
        // 记录当前置顶图层
        this._frontLayer = this._mapLayerListData[i];
        if (!this._frontLayer.isFront) {
          this._frontLayerOriginalIndex = this._frontLayer.zIndex;
          this._frontLayer.zIndex = this._zIndex + 2000;
          this._frontLayer.isFront = true;
          // 设置图层索引进行前置
          let l_layer = this._getLayer(this._frontLayer);
          l_layer.setZIndex(this._frontLayer.zIndex);
        }
        break;
      }
    }
  }
  // 置顶后重新排序
  this._layerCheckedList.sort(function (a, b) {
    let value1 = a["zIndex"];
    let value2 = b["zIndex"];
    return value2 - value1;
  });
};

/**
 * 设置图层上移
 * @param guid 当前图层的guid
 */
MapViewViewModel.prototype.raiseLayerByTreeId = function (guid) {
  for (let i = 0; i < this._layerCheckedList.length; i++) {
    if (this._layerCheckedList[i].guid === guid) {
      if (i !== 0) {
        // 上一个图层
        let layerConfig_s = this._layerCheckedList[i - 1];
        // 当前图层
        let layerConfig = this._layerCheckedList[i];
        // 当前图层和上一个都不为置顶图层时上移
        if (!layerConfig.isFront && !layerConfig_s.isFront) {
          // 上一个、当前图层属性
          let layer_s = this._getLayer(layerConfig_s);
          let layer = this._getLayer(layerConfig);
          // 图层zIndex交换
          let zIndex_s = layerConfig_s.zIndex;
          layerConfig_s.zIndex = layerConfig.zIndex;
          layerConfig.zIndex = zIndex_s;
          layer_s.setZIndex(layerConfig_s.zIndex);
          layer.setZIndex(layerConfig.zIndex);
          // 数组位置交换
          this._layerCheckedList[i] = layerConfig_s;
          this._layerCheckedList[i - 1] = layerConfig;
        }
      }
      break;
    }
  }
};
/**
 * 设置图层下移
 * @param id 当前图层的treeId
 */
MapViewViewModel.prototype.lowerLayerByTreeId = function (guid) {
  for (let i = 0; i < this._layerCheckedList.length; i++) {
    if (this._layerCheckedList[i].guid === guid) {
      if (i !== this._layerCheckedList.length - 1) {
        // 下一个图层
        let layerConfig_x = this._layerCheckedList[i + 1];
        // 当前图层
        let layerConfig = this._layerCheckedList[i];
        // 当前图层为置顶图层时不下移
        if (!layerConfig.isFront) {
          // 下一个、当前图层属性
          let layer_x = this._getLayer(layerConfig_x);
          let layer = this._getLayer(layerConfig);
          // 图层zIndex交换
          let zIndex_x = layerConfig_x.zIndex;
          layerConfig_x.zIndex = layerConfig.zIndex;
          layerConfig.zIndex = zIndex_x;
          layer_x.setZIndex(layerConfig_x.zIndex);
          layer.setZIndex(layerConfig.zIndex);
          // 数组位置交换
          this._layerCheckedList[i] = layerConfig_x;
          this._layerCheckedList[i + 1] = layerConfig;
        }
      }
      break;
    }
  }
};
/**
 * 还原所有图层顺序
 */
MapViewViewModel.prototype.resetLayerZIndex = function () {
  for (let i = 0; i < this._layerCheckedList.length; i++) {
    this._layerCheckedList[i].zIndex = this._layerCheckedList[i].zIndex_old;
    let _layer = this._getLayer(this._layerCheckedList[i]);
    if (_layer) _layer.setZIndex(this._layerCheckedList[i].zIndex);
  }
};

MapViewViewModel.prototype.pan = function () {
  this.map.stop();
  this.mapDraw.stopDrawState();
};

MapViewViewModel.prototype.fullExtent = function () {
  if (this.webMapInfo.bounds84) {
    this.map.fitBounds(this.webMapInfo.bounds84);
  } else {
    let center = this.mapOptions.center || this.webMapInfo.center;
    let zoom = this.mapOptions.zoom || this.webMapInfo.zoom;
    if (center && zoom) {
      this.map.setView(center, zoom);
    }
  }
};

/**
 * 拉框放大
 */
MapViewViewModel.prototype.startLKFD = function () {
  this.mapDraw.stopDrawRectPoint();
  this.mapDraw.startDrawRectPoint(
    points => {
      _resetMap(this.map, this.webMapInfo.scale, points, "fd");
    },
    true,
    false,
    "url(images/cur/cursor-tool-zoomout.cur),move"
  );
};
/**
 * 拉框缩小
 */
MapViewViewModel.prototype.startLKSX = function () {
  this.mapDraw.stopDrawRectPoint();
  this.mapDraw.startDrawRectPoint(
    points => {
      _resetMap(this.map, this.webMapInfo.scale, points, "sx");
    },
    true,
    false,
    "url(images/cur/cursor-tool-zoomin.cur),move"
  );
};

MapViewViewModel.prototype.showBaseLayer = function (layerName, visible) {
  if (!this._props.baseLayerInfo) return;

  let _layer = this._mapBaseLayerData.get(layerName);

  if (!this._mapBaseLayerData.get(layerName)) {
    if (layerName === "sl") {
      if (typeof this._props.baseLayerInfo.sl === "undefined") return;
      _layer = this.createBaseLayer(this._props.baseLayerInfo.sl);
    } else if (layerName === "yx") {
      if (typeof this._props.baseLayerInfo.yx === "undefined") return;
      _layer = this.createBaseLayer(this._props.baseLayerInfo.yx);
    }
    this._mapBaseLayerData.set(layerName, _layer);
  }
  if (visible) {
    if (_layer && !this.map.hasLayer(_layer.layer)) {
      _layer.layer.addTo(this.map);
      _layer.visible = true;
    }
  } else {
    if (_layer && this.map.hasLayer(_layer.layer)) {
      _layer.layer.remove();
      _layer.visible = false;
    }
  }
};
MapViewViewModel.prototype.createBaseLayer = function (config) {
  let layers = new L.layerGroup();
  let parames = [];
  if (config.layers !== undefined) {
    if (Array.isArray(config.layers)) {
      parames = config.layers;
    } else {
      parames = [config.layers];
    }
  }
  for (let i = parames.length - 1; i >= 0; i--) {
    let layer = null;
    switch (parames[i].type) {
      case "tianditu": // 天地图
        layer = L.KqGIS.tiandituTileLayer(parames[i]);
        break;
      // case "gaode": // 高德地图
      //   layer = L.kqmap.mapping.gaodeTileLayer({ layerType: parames[i]["layer"] });
      //   break;
      // case "tencent": // 腾讯地图
      //   let layerName = parames[i]["layer"];
      //   if (layerNames[i] === "vec") {
      //     layerName = "map_tiles";
      //   } else if (layerNames[i] === "img") {
      //     layerName = "sate_tiles";
      //   }
      //   layer = L.kqmap.mapping.tencentTileLayer({ layerType: layerName });
      //   break;
      // case "baidu": // 百度地图
      //   layer = L.kqmap.mapping.baiduTileLayer({ layerType: parames[i]["layer"] });
      //   break;
      // case "google": // 谷歌地图在线
      //   layer = L.kqmap.mapping.googleTileLayer({ layerType: parames[i]["layer"] });
      //   break;
      // case "geoq": // 智图
      //   layer = L.kqmap.mapping.geoqTileLayer({ layerType: parames[i]["layer"] });
      //   break;
      // case "googlelx": // 谷歌地图离线
      //   layer = L.tileLayer(parames[i]["url"], { maxZoom: parames[i]["maxZoom"] });
      //   break;
      case "wmts": // wmts
        let params = {
          layer: parames[i]["layer"],
          tilematrixSet: parames[i]["tilematrixSet"] === undefined ? "default028mm" : parames[i]["tilematrixSet"],
          style: parames[i]["style"] === undefined ? "default" : parames[i]["style"],
          format: parames[i]["format"] === undefined ? "image/jpgpng" : parames[i]["format"],
          transparent: parames[i]["transparent"] === undefined ? true : parames[i]["transparent"],
          zoomOffset:
            parames[i]["zoomOffset"] === undefined ? this.mapViewParams.zoomOffset : Number(parames[i]["zoomOffset"])
        };
        if (parames[i]["subdomains"] !== undefined) {
          params["subdomains"] = parames[i]["subdomains"];
        }
        let url = parames[i]["url"];
        if (parames[i]["token"] !== undefined) {
          url += "?" + parames[i]["token"];
        }
        layer = window.L.kqmap.mapping.wmtsLayer(url, params);
        break;
    }
    if (layer !== null) {
      layers.addLayer(layer);
    }
  }
  return {
    layer: layers,
    title: config.title,
    parames: parames
  };
};

/**
 * 生成卷帘
 * @param {Object} layer - 选中需要卷帘的图层，不传时会清除卷帘
 */
MapViewViewModel.prototype.initRoller = function (layer) {
  this.mapControl.initRoller(layer);
};

/**
 * 生成滤镜
 * @param {Object} config
 * @param {number} [config.radius=100] - 放大镜的半径，以像素为单位
 * @param {number} [config.zoomOffset=3] - 主地图缩放和放大镜之间的缩放级别偏移
 * @param {number} [config.fixedZoom= -1] - 如果不同于-1，则定义始终在放大镜中使用的固定缩放级别，忽略主地图缩放和zoomOffet值。
 * @param {Boolean} [config.fixedPosition= false] - 如果true，放大镜将停留在地图上的同一位置，而不是跟随鼠标光标。
 * @param {array} latLng - 放大镜的初始位置，无论是在主地图上还是作为放大视图的中心。如果fixedPosition是true，它会一直保持这个位置。
 * @param {array} layers - 要在放大视图中显示的图层集
 */
MapViewViewModel.prototype.createFilter = function (config) {
  !config.latLng ? delete config.latLng : null;
  return this.mapControl.createFilter(config);
};

/**
 * 清除滤镜
 * @param {object} layer - 要清除的滤镜图层
 */
MapViewViewModel.prototype.clearFilter = function (layer) {
  this.mapControl.clearFilter(layer);
};

/**
 * 生成比例尺并返回
 * @param {Object} config
 * @param {string|boolean} [config.fill=false] - 填充，false/'nofill'|'fill'|'hollow'|'line',默认 false
 * @param {boolean} [config.doubleLine=false] - 双线模式，默认false关闭
 * @param {boolean} [config.showSubunits=false] - 显示子单元,在零的左侧显示较小的分区,默认false关闭
 * @param {number} [config.minUnitWidth = 30] - 比例单位的最小宽度
 * @param {number} [config.maxUnitsWidth = 240] - 不包含子单位的比例尺的最大宽度
 * @param {string} [config.labelPlacement = 'auto'] - 'auto'|'top'|'bottom'，在比例尺的顶部/底部显示距离标签。如果设置为自动，当比例控制在地图底部时，标签将被放置在顶部，当比例控制在地图顶部时，标签将被放置在底部（position参数）
 * @param {string} [config.position = 'bottomleft'] - 控件的位置（地图角之一）。可能的值为'topleft', 'topright','bottomleft'或'bottomright'
 * @param {boolean} [config.updateWhenIdle= false] - 如果true，控件在 地图moveend后更新，否则它在地图move时更新。
 */
MapViewViewModel.prototype.createScale = function (config) {
  return this.mapControl.createScale(config);
};

/**
 * 设置新地图信息 重新初始化
 * @param webMapInfo {object}
 */
MapViewViewModel.prototype.reInitMap = function (webMapInfo) {
  this.webMapInfo = webMapInfo;

  this._initWebMap();
};

/**
 * 添加临时数据
 * 文件导入shp、csv、kml等解析成GeoJSON格式传入，支持properties带颜色
 * 添加数据的类型-geojson(GeoJSON数据) -kqserver(kq服务的数据) -wmts -wms
 * @param layers {Array}
 * @param isshow {boolean} 是否默认显示
 * @param layerList {Array} 服务是kqserver时，layers参数传入的树树形结构，需要传入参数layerList，标识其中哪些节点是图层数据
 */
MapViewViewModel.prototype.addTempData = function (layers, isshow, layerList = null) {
  // layers = [
  //   {
  //     name: "t1",
  //     guid: "fsdf3w536sqw23",
  //     data: {
  //       type: "FeatureCollection",
  //       features: [
  //         {
  //           type: "Feature",
  //           properties: {
  //             adcode: 420102,
  //             name: "江岸区",
  //             color: "#FCC6D0",
  //             center: [114.30304, 30.594911],
  //             centroid: [114.320201, 30.643422],
  //             childrenNum: 0,
  //             level: "district",
  //             parent: { adcode: 420100 },
  //             subFeatureIndex: 0,
  //             acroutes: [100000, 420000, 420100]
  //           },
  //           geometry: {
  //             type: "MultiPolygon",
  //             coordinates: [
  //               [
  //                 [
  //                   [114.303197, 30.574275],
  //                   [114.297952, 30.576267],
  //                   [114.287958, 30.584154],
  //                   [114.282948, 30.587694],
  //                   [114.27381, 30.591498],
  //                   [114.271588, 30.59689],
  //                   [114.278448, 30.599732],
  //                   [114.285639, 30.606697],
  //                   [114.28114, 30.611183],
  //                   [114.277371, 30.616476],
  //                   [114.275384, 30.621254],
  //                   [114.272016, 30.625781],
  //                   [114.257937, 30.642271],
  //                   [114.267682, 30.64766],
  //                   [114.272609, 30.652283],
  //                   [114.282106, 30.659871],
  //                   [114.288428, 30.66402],
  //                   [114.292955, 30.670049],
  //                   [114.29243, 30.675548],
  //                   [114.289559, 30.681965],
  //                   [114.289504, 30.684164],
  //                   [114.292637, 30.681701],
  //                   [114.296157, 30.676494],
  //                   [114.297883, 30.674893],
  //                   [114.30426, 30.675144],
  //                   [114.308746, 30.677497],
  //                   [114.315302, 30.682438],
  //                   [114.318863, 30.683608],
  //                   [114.323763, 30.682466],
  //                   [114.329464, 30.674921],
  //                   [114.333964, 30.673613],
  //                   [114.338615, 30.675408],
  //                   [114.340796, 30.680128],
  //                   [114.343957, 30.682856],
  //                   [114.352405, 30.68802],
  //                   [114.355552, 30.688869],
  //                   [114.359693, 30.691082],
  //                   [114.363847, 30.692043],
  //                   [114.365545, 30.693393],
  //                   [114.369162, 30.698417],
  //                   [114.37126, 30.698807],
  //                   [114.38371, 30.695021],
  //                   [114.392047, 30.69555],
  //                   [114.396643, 30.691361],
  //                   [114.399487, 30.686183],
  //                   [114.4028, 30.68326],
  //                   [114.394601, 30.680768],
  //                   [114.388458, 30.67758],
  //                   [114.383241, 30.674225],
  //                   [114.374835, 30.666707],
  //                   [114.368651, 30.660512],
  //                   [114.360755, 30.652018],
  //                   [114.351218, 30.642855],
  //                   [114.341486, 30.628079],
  //                   [114.335675, 30.619262],
  //                   [114.329519, 30.612883],
  //                   [114.309298, 30.583848],
  //                   [114.303197, 30.574275]
  //                 ]
  //               ]
  //             ]
  //           }
  //         }
  //       ]
  //     }
  //   }
  // ];
  let haslslayer = false,
    lslayer = null;
  // 在图层树创建临时节点
  for (let i = 0; i < this._mapLayerTreeData.length; i++) {
    if (this._mapLayerTreeData[i].name === "临时图层") {
      haslslayer = true;
      lslayer = this._mapLayerTreeData[i];
      break;
    }
  }
  if (!haslslayer) {
    lslayer = {
      name: "临时图层",
      guid: "134f6h768tha23",
      children: []
    };
    this._mapLayerTreeData.push(lslayer);
  }
  // 临时节点添加数据
  let keys = [];
  layers.forEach(item => {
    item.lsType = "ls";
    item.serverType = "geojson";
    lslayer.children.push(item);
  });
  // 图层集合添加图层数据
  if (layerList) {
    layerList.forEach(item => {
      item.lsType = "ls";
      this._mapLayerListData.push(item);
      keys.push(item.guid);
    });
  } else {
    layers.forEach(item => {
      item.lsType = "ls";
      this._mapLayerListData.push(item);
      keys.push(item.guid);
    });
  }
  // 添加后直接显示
  if (isshow) {
    this.setLayerVisibleStateByIds(keys, true);
  } else {
    // 广播数据更新了
    this.fire("resetLayerDatas");
  }
};
/**
 * 删除临时数据
 * ???清除所有 需要考虑部分清除
 * @param args
 */
MapViewViewModel.prototype.delTempData = function (args) {
  // 删除 临时图层勾选 使用setLayerVisibleStateByIds取消勾选（会删除存储的layer对象）
  // 删除 _mapLayerTreeData的临时数据节点
  // 删除 _mapLayerListData的临时图层
  if (!args) {
    // 在图层树创建临时节点
    let i,
      keys = [];
    for (i = 0; i < this._mapLayerTreeData.length; i++) {
      if (this._mapLayerTreeData[i].name === "临时图层") {
        this._mapLayerTreeData.splice(i, 1);
        break;
      }
    }
    for (i = 0; i < this._mapLayerListData.length; i++) {
      if (this._mapLayerListData[i].lsType === "ls") {
        keys.push(this._mapLayerListData.guid);
      }
    }

    this.setLayerVisibleStateByIds(keys, false);

    for (let i = 0; i < this._mapLayerListData.length; i++) {
      if (this._mapLayerListData[i].lsType === "ls") {
        this._mapLayerListData.splice(i, 1);
      }
    }
  }
};

MapViewViewModel.prototype.showOnlineBaseLayer = function (layerConfig, visible) {
  if (!this._mapOnLineLayerGroup) {
    this._mapOnLineLayerGroup = new window.L.layerGroup().addTo(this.map);
  }

  let layer = this._mapOnLineLayerData.get(layerConfig.guid);
  if (!layer) {
    layer = this._createLayer(layerConfig);
    this._mapOnLineLayerData.set(layerConfig.guid, layer);
  }
  if (visible) {
    if (!this._mapOnLineLayerGroup.hasLayer(layer)) this._mapOnLineLayerGroup.addLayer(layer);
  } else {
    this._mapOnLineLayerGroup.removeLayer(layer);
  }
};
MapViewViewModel.prototype.removeAllOnlineBaseLayer = function () {
  if (this._mapOnLineLayerGroup) this._mapOnLineLayerGroup.clearLayers();
};

/**
 * 地图范围计算与操作
 */
function _resetMap(map, scale, geoJson, zoomType) {
  if (geoJson.type === "Point") return;

  let mapBounds = map.getBounds();
  let level = map.getZoom();
  //let ss = this.scales;
  let ss = scale;
  let _width_map = mapBounds._northEast.lng - mapBounds._southWest.lng;
  let _height_map = mapBounds._northEast.lat - mapBounds._southWest.lat;

  let point1 = geoJson.coordinates[0][0];
  let point2 = geoJson.coordinates[0][2];
  let _width_geo = Math.abs(point1[0] - point2[0]);
  let _height_geo = Math.abs(point1[1] - point2[1]);
  // 获取map的宽高除以拉框宽高的倍数 取最小值
  let a = _width_map / _width_geo,
    b = _height_map / _height_geo;
  let c = a < b ? a : b;

  let d = 0;
  let levelNEW = 0;
  if (zoomType === "fd") {
    d = ss[level] / c;
    for (let i = level; i < ss.length; i++) {
      if (d >= ss[i] || i === ss.length - 1) {
        levelNEW = i;
        break;
      }
    }
    if (levelNEW > level) levelNEW--;
    if (levelNEW === level) levelNEW++;
  } else {
    d = ss[level] * c;
    for (let i = level; i >= 0; i--) {
      if (d <= ss[i]) {
        levelNEW = i;
        break;
      }
    }
    if (levelNEW < level) levelNEW++;
    if (levelNEW === level && levelNEW !== 0) levelNEW--;
  }
  map.setView([(point1[1] + point2[1]) / 2, (point1[0] + point2[0]) / 2], levelNEW);
}
