import { utils, mapLayerUtils, mapViewUtils } from "@/common/_utils/gis-utils";

export default class RollerViewModel extends L.Evented {
  _props = null;
  webMap = null;
  constructor(mapTarget) {
    super();
    this._props = { mapTarget };
    // 事件绑定this
    this._loadedAsync = this._loadedAsync.bind(this);
    this._loadedSync = this._loadedSync.bind(this);
    this._updateLayerData = this._updateLayerData.bind(this);
    this.setLayerData = this.setLayerData.bind(this);
    this._registerEvents();
  }
  /**
   * 注册事件
   */
  _registerEvents() {
    // 监听地图初始化完成事件
    // 异步
    mapViewUtils.bindViewEvents("mounted", this._loadedAsync, this._props.mapTarget);
    mapViewUtils.bindViewEvents("resetLayerDatas", this._updateLayerData, this._props.mapTarget);
    // 同步
    utils.getWebMap(this._props.mapTarget, this._loadedSync);
  }

  // 图层监听事件
  _loadedAsync(e) {
    // 如果图层变化事件是自己触发的 则不继续执行 避免死循环
    if (e.eventTarget && e.eventTarget === this) {
      return;
    }
    this.webMap = e.target;
    this.setLayerData();
  }

  _loadedSync(e) {
    this.webMap = e;
    this.setLayerData();
  }

  // 设置图层树
  setLayerData() {
    let maplayers = mapLayerUtils.getLayerCheckedList(this._props.mapTarget);
    if (maplayers && maplayers.length > 0) {
      this.fire("setData", { maplayers });
      return { maplayers };
    }
  }

  // 更新图层树
  _updateLayerData() {
    let maplayers = mapLayerUtils.getLayerCheckedList(this._props.mapTarget);
    this.fire("updateData", { maplayers });
  }
  /**
   * 生成卷帘
   * @param {Object} layer - 选中需要卷帘的图层，不传时会清除卷帘
   */
  initRoller(layer) {
    this.webMap.initRoller(layer);
  }

  destroy() {
    this.off();
  }
}
