import { onMounted, onUnmounted, defineComponent } from "vue";
import ScaleViewModel from "./ScaleViewModel";

export default defineComponent({
  name: "Kq2dScale",
  props: {
    mapTarget: String,
    // 填充，false|'fill'|'hollow'|'line',默认 false
    fill: {
      type: [Boolean, String],
      default: false
    },
    // 双线模式，默认false关闭
    doubleLine: {
      type: Boolean,
      default: false
    },
    // 显示子单元,在零的左侧显示较小的分区,默认false关闭
    showSubunits: {
      type: Boolean,
      default: false
    },
    // 比例单位的最小宽度
    minUnitWidth: {
      type: Number,
      default: 30
    },
    // 不包含子单位的比例尺的最大宽度
    maxUnitsWidth: {
      type: Number,
      default: 240
    },
    // 'auto'|'top'|'bottom'，在比例尺的顶部/底部显示距离标签。如果设置为自动，当比例控制在地图底部时，标签将被放置在顶部，当比例控制在地图顶部时，标签将被放置在底部（position参数）
    labelPlacement: {
      type: String,
      default: "auto"
    },
    // 控件的位置（地图角之一）。可能的值为'topleft', 'topright','bottomleft'或'bottomright'
    position: {
      type: String,
      default: "topright"
    },
    // 如果true，控件在 moveend后更新，否则它move是更新
    updateWhenIdle: {
      type: Boolean,
      default: false
    },
    // 设置scale样式的回调函数
    setScaleStyle: Function
  },
  setup(props, context) {
    let viewModel = null;
    const {
      mapTarget,
      fill,
      doubleLine,
      showSubunits,
      minUnitWidth,
      maxUnitsWidth,
      labelPlacement,
      position,
      updateWhenIdle
    } = props;

    // 创建scale
    const createScale = webMap => {
      return webMap.createScale({
        fill,
        doubleLine,
        showSubunits,
        minUnitWidth,
        maxUnitsWidth,
        labelPlacement,
        position,
        updateWhenIdle
      });
    };

    const setScale = () => {
      return new Promise(resolve => {
        if (viewModel.webMap && viewModel.webMap.createScale) {
          // 同步生成
          let scale = createScale(viewModel.webMap);
          resolve(scale);
        } else {
          // 异步生成
          viewModel.on("getWebMap", _data => {
            let scale = createScale(_data.webMap);
            resolve(scale);
          });
        }
      });
    };

    // 初始化
    const init = () => {
      const { setScaleStyle } = props;
      setScale().then(result => {
        setScaleStyle && setScaleStyle(result);
      });
    };

    // 清除比例尺
    const clear = () => {
      viewModel.destroy();
    };    

    onMounted(() => {
      viewModel = new ScaleViewModel(mapTarget);     
      init();
    });

    onUnmounted(() => {
      clear();
      viewModel = null;
    });

    return {     
      clear
    };
  },
  render() {
    return null;
  }
});
