import L from "leaflet";
import { convertXY } from "@/common/_utils/util";
import { utils } from "@/common/_utils/gis-utils";

export default class StatusBarViewModel extends L.Evented {
  _webMap = null;

  constructor(options) {
    super();

    this._props = options.props;

    this._loaded = this._loaded.bind(this);

    this._onMousemove = this._onMousemove.bind(this);

    this._onZoomend = this._onZoomend.bind(this);

    this._registerEvents();
  }

  /**
   * 注册事件
   */
  _registerEvents() {
    utils.getWebMap(this._mapTarget, this._loaded);
  }
  _loaded(webmap) {
    this._webMap = webmap;

    this._setDefaultData();
  }

  _setDefaultData() {
    if (!this._webMap.map) return;

    this._webMap.map.on("mousemove", this._onMousemove); // 鼠标移动事件
    this._webMap.map.on("zoomend", this._onZoomend); // 鼠标缩放事件

    let center = this._webMap.map.getCenter();

    this._setXY(center);

    this._setLevel();
  }

  _onMousemove(e) {
    this._setXY(e.latlng);
  }

  _setXY(point) {
    let latFixed, lngFixed, lngName, latName;

    let epsg = this._webMap.webMapInfo.spatialReference.epsg;

    if (!(epsg === 4490 || epsg === 4326)) {
      let firstProjection = new proj4.Proj("EPSG:4326");
      let secondProjection = new proj4.Proj("EPSG:" + epsg);
      let result = proj4.transform(firstProjection, secondProjection, [point.lng, point.lat]);

      latFixed = result.y.toFixed(2);
      lngFixed = result.x.toFixed(2);
      lngName = "经度：";
      latName = "纬度：";
    } else {
      latFixed = convertXY(point.lat);
      lngFixed = convertXY(point.lng);
      lngName = "X：";
      latName = "Y：";
    }

    this.fire("setXY", { lng: lngFixed, lat: latFixed, lngName, latName });
  }
  _onZoomend(e) {
    this._setLevel();
  }

  _setLevel() {
    let value = "";
    if (this._props.position && this._props.position === "bottom-center") {
      value += "丨";
    }
    value += "级别：" + this._webMap.map.getZoom();

    this.fire("setLevel", { level: value });
  }

  destroy() {
    // 删除所有事件监听
    this.off();

    this._webMap.map.off("mousemove", this._onMousemove);
    this._webMap.map.off("zoomend", this._onZoomend);
  }
}
